@@ -45,25 +45,24 @@ use Exporter qw( import );
 use File::ShareDir;
 use File::Spec;
 use Log::Any::Adapter;
-use Mason;
 use Params::Validate qw( :all );
 
 
 
 =head1 NAME
 
-App::MFILE::WWW - generic web front-end with demo app
+App::MFILE::WWW - Generic web front-end with demo app
 
 
 
 
 =head1 VERSION
 
-Version 0.079
+Version 0.088
 
 =cut
 
-our $VERSION = '0.079';
+our $VERSION = '0.088';
 
 
 
@@ -81,13 +80,14 @@ directory. The license text is also reprodued at the top of each source file.
 =head1 DESCRIPTION
 
 This distro contains a generic framework for developing web front-ends to 
-REST resources. The framework includes a web server (based on Plack and
-L<Web::Machine>), CSS and HTML for the "frame" where the application
-displays its screens, and "widgets" for defining the application's login
-dialog, menus, forms, and actions. 
+REST resources. The framework consists of a web server based on Plack and
+L<Web::Machine>, CSS and HTML for the "app frame" (on-screen area where the
+application's "screens" are displayed), and "widgets" for defining the
+application's login dialog, menus, forms, and actions. 
 
-For illustration, the distro contains a demo app that works with
-L<App::Dochazka::REST>.
+For illustration, the distro contains a demo app that authenticates against
+L<App::Dochazka::REST> and contains a single menu, a simple form, and a
+sample action.
 
 
 =head1 INTRODUCTION
@@ -117,9 +117,9 @@ interface to the REST server.
 
 =item * WWW client/frontend
 
-The WWW frontend, based on this distro, is a web server that serves HTML,
-CSS, and JavaScript code to users for a menu-driven "browser experience" of
-the application.
+The WWW frontend, built based on this distro, is a web server that serves HTML,
+CSS, and JavaScript code to users to provide them with a menu-driven "browser
+experience" of the application.
 
 =back
 
@@ -127,110 +127,85 @@ Conceptually, the clients act as proxies between the user and the REST
 server. Taking this one step further, the REST server itself is a proxy between
 the client and the database engine.
 
-The philosophy behind this design is that you, the user, have the freedom and
-the flexibility to write your own client, on any platform, in any language --
-however you see fit. 
+The philosophy behind this RESTful design is that you, the user, have the
+freedom and the flexibility to write your own client, on any platform, in any
+language -- however you see fit. From a technical perspective, the strict
+separation between the server and clients makes the application as a whole more
+robust.
 
 
-=head1 MAJOR COMPONENTS
+=head1 REQUEST-RESPONSE CYCLE
 
-The web frontend is implemented using the following technologies:
-
-=over
-
-=item * B<Perl> - server-side code
-
-=item * B<nginx> - reverse proxy (optional)
-
-=item * L<HTTP::Server::PSGI>, L<Starman>, or other PSGI-compatible HTTP server
-
-=item * L<Plack> - interface with HTTP server 
-
-=item * L<Web::Machine> - HTTP request/response cycle
-
-=item * L<App::MFILE::WWW::Resource> - overlays L<Web::Machine::Resource>
-
-=item * HTML and CSS code served to the user's browser
-
-=item * L<JavaScript> code served to and running in the user's browser
-
-=item * AJAX calls from the JavaScript code back to the web server, which
-forwards them to the REST server and sends the results back to the
-JavaScript side - see the next section for details
-
-=back
-
-
-=head2 Request-response cycle
-
-A more detailed description of the request-response cycle implemented by
-L<App::MFILE::WWW>:
+The HTTP request-response cycle is implemented as follows:
 
 =over
 
 =item * B<nginx> listens for incoming connections on port 80/443 of the server
 
 =item * When a connection comes in, B<nginx> decrypts it and forwards it to a
-high-numbered port where Starman is listening
+high-numbered port where a PSGI-compatible HTTP server (such as L<Starman>) is
+listening
 
-=item * Starman takes the connection and passes it to the Plack middleware
-
-=item * the key middleware component is L<Plack::Middleware::Session>, which
-assigns an ID to the session, maintains whatever data the server-side
-code needs to associate with the session, links the session to the user's
-browser via a cookie, and provides the application a hook (in the Plack
-environment that is included in the HTTP request) to access the session data
+=item * The HTTP server takes the connection and passes it to the Plack middleware.
+The key middleware component is L<Plack::Middleware::Session>, which assigns an
+ID to the session, stores whatever data the server-side code needs to associate
+with the session, links the session to the user's browser via a cookie, and
+provides the application a hook (in the Plack environment stored in the HTTP
+request) to access the session data
 
 =item * if the connection is asking for static content (defined as anything in
-C<images/>, C<css/>, or C<js/>), Plack and Starman serve that content
-immediately and the request doesn't even make it into our code
+C<images/>, C<css/>, or C<js/>), that content is served immediately and the
+request doesn't even make it into our Perl code
 
 =item * any other path is considered dynamic content and is passed to
 L<Web::Machine> for processing -- L<Web::Machine> implements the HTTP standard
 as a state machine
 
 =item * the L<Web::Machine> state machine takes the incoming request and runs
-it through several functions that are implemented in
-L<App::MFILE::WWW::Resource> -- for example, 'malformed_request' examines
-the request body, if any, and if it is not valid JSON, the HTTP server returns
-'400 Malformed Request'
+it through several functions that are overlayed in L<App::MFILE::WWW::Resource>
+- an appropriate HTTP error code is returned if the request doesn't make it
+through the state machine. Along the way, log messages are written to the log.
 
 =item * as part of the state machine, all incoming requests are subject to
-"authorization" (in the HTTP sense), which actually means authentication.
+"authorization" (in the HTTP sense, which actually means authentication).
 First, the session data is examined to determine if the request belongs to an
-authorized session. If it doesn't, the request is treated as a login/logout
-attempt.
+existing authorized session. If it doesn't, the request is treated as a
+login/logout attempt -- the session is cleared and control passes to the 
+JavaScript side, which, lacking a currentUser object, displays the login
+dialog.
 
 =item * once an authorized session is established, there are two types of
 requests: GET and POST
 
-=item * incoming GET requests can happen when the page is reloaded -
-in an authorized session, this causes the main menu to be displayed. JavaScript
-form buffers and other data structures are reset to their default values. Note
-that L<App::MFILE::WWW> pays no attention to the URI - if the user enters
-a path (e.g. http://mfile.site/some/bogus/path), this will be treated like
-any other page reload.
+=item * incoming GET requests happen whenever the page is reloaded -
+in an authorized session, this causes the main menu to be displayed, but all
+static content (CSS and JavaScript modules) are reloaded for a "clean slate",
+as if the user had just logged in.
+
+=item * Note that L<App::MFILE::WWW> pays no attention to the URI - if the user
+enters a path (e.g. http://mfile.site/some/bogus/path), this will be treated
+like any other page (re)load and the path is simply ignored.
 
 =item * if the session is expired or invalid, any incoming GET request will
 cause the login dialog to be displayed.
 
 =item * well-formed POST requests are assumed to be AJAX calls and are
-processed by the C<process_post> routine. First, the request body is examined.
-The request body of all AJAX calls must adhere to a simple structure:
+directed to the C<process_post> routine, which first examines the request body, 
+which must adhere to a simple structure:
 
     { method: "GET", path: "employee/current", body: { ... } }
 
-where 'method' is either GET or POST, 'path' is the path to the REST server
-resource being requested, and 'body' is the request body to be forwarded 
-to the REST server. Provided the request is properly authorized and the
-body is well-formed, the request is forwarded to the REST server and the 
-REST server's response is sent back to the user's browser, where it is
-processed by the JavaScript code.
+where 'method' is any HTTP method accepted by the REST server, 'path' is a
+valid path to a REST server resource, and 'body' is the content body to be
+sent in the HTTP request to the REST server. Provided the request is properly
+authorized and the body is well-formed, the request is forwarded to the REST
+server via the L<App::MFILE> package's C<rest_req> routine and the REST
+server's response is sent back to the user's browser, where it is processed by
+the JavaScript code.
 
 =item * under ordinary operation, the user will spend 99% of her time
 interacting with the JavaScript code running in her browser, which will
-communicate asynchronously with the REST server via AJAX calls as described
-above.
+communicate asynchronously as needed with the REST server via AJAX calls.
 
 =back
 
@@ -267,8 +242,9 @@ my $dist_dir = File::ShareDir::dist_dir( 'App-MFILE-WWW' );
 
 =head2 init
 
-Initialization routine - runs when the server is started. Loads configuration
-files from the distro and site configuration directories, and sets up logging.
+Initialization routine - run from C<bin/mfile-www>, the server startup script.
+This routine loads configuration parameters from files in the distro and site
+configuration directories, and sets up logging.
 
 =cut
 
